@@ -1,5 +1,6 @@
-use crate::style::TextStyle;
-use std::{iter, path::PathBuf, str::FromStr};
+use super::text_style::TextStyle;
+use comrak::nodes::AlertType;
+use std::{fmt, iter, path::PathBuf, str::FromStr};
 use unicode_width::UnicodeWidthStr;
 
 /// A markdown element.
@@ -12,13 +13,13 @@ pub(crate) enum MarkdownElement {
     FrontMatter(String),
 
     /// A setex heading.
-    SetexHeading { text: TextBlock },
+    SetexHeading { text: Line },
 
     /// A normal heading.
-    Heading { level: u8, text: TextBlock },
+    Heading { level: u8, text: Line },
 
     /// A paragraph composed by a list of lines.
-    Paragraph(Vec<TextBlock>),
+    Paragraph(Vec<Line>),
 
     /// An image.
     Image { path: PathBuf, title: String, source_position: SourcePosition },
@@ -50,19 +51,29 @@ pub(crate) enum MarkdownElement {
     Comment { comment: String, source_position: SourcePosition },
 
     /// A block quote containing a list of lines.
-    BlockQuote(Vec<TextBlock>),
+    BlockQuote(Vec<Line>),
+
+    /// An alert.
+    Alert {
+        /// The alert's type.
+        alert_type: AlertType,
+
+        /// The optional title.
+        title: Option<String>,
+
+        /// The content lines in this alert.
+        lines: Vec<Line>,
+    },
 }
 
-#[derive(Clone, Debug, Default)]
-pub(crate) struct SourcePosition {
+#[derive(Clone, Copy, Debug, Default)]
+pub struct SourcePosition {
     pub(crate) start: LineColumn,
 }
 
-impl SourcePosition {
-    pub(crate) fn offset_lines(&self, offset: usize) -> SourcePosition {
-        let mut output = self.clone();
-        output.start.line += offset;
-        output
+impl fmt::Display for SourcePosition {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(f, "{}:{}", self.start.line, self.start.column)
     }
 }
 
@@ -72,7 +83,7 @@ impl From<comrak::nodes::Sourcepos> for SourcePosition {
     }
 }
 
-#[derive(Clone, Debug, Default)]
+#[derive(Clone, Copy, Debug, Default)]
 pub(crate) struct LineColumn {
     pub(crate) line: usize,
     pub(crate) column: usize,
@@ -84,13 +95,13 @@ impl From<comrak::nodes::LineColumn> for LineColumn {
     }
 }
 
-/// A block of text.
+/// A text line.
 ///
 /// Text is represented as a series of chunks, each with their own formatting.
 #[derive(Clone, Debug, PartialEq, Eq, Default)]
-pub(crate) struct TextBlock(pub(crate) Vec<Text>);
+pub(crate) struct Line(pub(crate) Vec<Text>);
 
-impl TextBlock {
+impl Line {
     /// Get the total width for this text.
     pub(crate) fn width(&self) -> usize {
         self.0.iter().map(|text| text.content.width()).sum()
@@ -104,7 +115,7 @@ impl TextBlock {
     }
 }
 
-impl<T: Into<Text>> From<T> for TextBlock {
+impl<T: Into<Text>> From<T> for Line {
     fn from(text: T) -> Self {
         Self(vec![text.into()])
     }
@@ -147,7 +158,7 @@ pub(crate) struct ListItem {
     pub(crate) depth: u8,
 
     /// The contents of this list item.
-    pub(crate) contents: TextBlock,
+    pub(crate) contents: Line,
 
     /// The type of list item.
     pub(crate) item_type: ListItemType,
@@ -185,7 +196,7 @@ impl Table {
     /// Iterates all the text entries in a column.
     ///
     /// This includes the header.
-    pub(crate) fn iter_column(&self, column: usize) -> impl Iterator<Item = &TextBlock> {
+    pub(crate) fn iter_column(&self, column: usize) -> impl Iterator<Item = &Line> {
         let header_element = &self.header.0[column];
         let row_elements = self.rows.iter().map(move |row| &row.0[column]);
         iter::once(header_element).chain(row_elements)
@@ -194,7 +205,7 @@ impl Table {
 
 /// A table row.
 #[derive(Clone, Debug, PartialEq, Eq)]
-pub(crate) struct TableRow(pub(crate) Vec<TextBlock>);
+pub(crate) struct TableRow(pub(crate) Vec<Line>);
 
 /// A percentage.
 #[derive(Clone, Debug, PartialEq, Eq)]
