@@ -1,15 +1,15 @@
-use super::elements::SourcePosition;
-use crate::{
-    markdown::elements::{ListItem, ListItemType, MarkdownElement, Table, TableRow, Text, TextBlock},
-    style::TextStyle,
+use super::{
+    elements::{Line, ListItem, ListItemType, MarkdownElement, SourcePosition, Table, TableRow, Text},
+    html::{HtmlInline, HtmlParser, ParseHtmlError},
+    text_style::TextStyle,
 };
 use comrak::{
     Arena, ComrakOptions,
     arena_tree::Node,
     format_commonmark,
     nodes::{
-        Ast, AstNode, ListDelimType, ListType, NodeCodeBlock, NodeHeading, NodeHtmlBlock, NodeList, NodeValue,
-        Sourcepos,
+        Ast, AstNode, ListDelimType, ListType, NodeAlert, NodeCodeBlock, NodeHeading, NodeHtmlBlock, NodeList,
+        NodeValue, Sourcepos,
     },
     parse_document,
 };
@@ -22,7 +22,7 @@ use std::{
 /// The result of parsing a markdown file.
 pub(crate) type ParseResult<T> = Result<T, ParseError>;
 
-struct ParserOptions(ComrakOptions<'static>);
+struct ParserOptions(comrak::Options<'static>);
 
 impl Default for ParserOptions {
     fn default() -> Self {
@@ -31,6 +31,7 @@ impl Default for ParserOptions {
         options.extension.table = true;
         options.extension.strikethrough = true;
         options.extension.multiline_block_quotes = true;
+        options.extension.alerts = true;
         Self(options)
     }
 }
@@ -40,7 +41,7 @@ impl Default for ParserOptions {
 /// This takes the contents of a markdown file and parses it into a list of [MarkdownElement].
 pub struct MarkdownParser<'a> {
     arena: &'a Arena<AstNode<'a>>,
-    options: ComrakOptions<'static>,
+    options: comrak::Options<'static>,
 }
 
 impl<'a> MarkdownParser<'a> {
@@ -53,39 +54,13 @@ impl<'a> MarkdownParser<'a> {
     pub(crate) fn parse(&self, contents: &str) -> ParseResult<Vec<MarkdownElement>> {
         let node = parse_document(self.arena, contents, &self.options);
         let mut elements = Vec::new();
-        let mut lines_offset = 0;
         for node in node.children() {
-            let mut parsed_elements =
-                self.parse_node(node).map_err(|e| ParseError::new(e.kind, e.sourcepos.offset_lines(lines_offset)))?;
-            if let Some(MarkdownElement::FrontMatter(contents)) = parsed_elements.first() {
-                lines_offset += contents.lines().count() + 2;
-            }
-            // comrak ignores the lines in the front matter so we need to offset this ourselves.
-            Self::adjust_source_positions(parsed_elements.iter_mut(), lines_offset);
+            let parsed_elements = self.parse_node(node).map_err(|e| ParseError::new(e.kind, e.sourcepos))?;
             elements.extend(parsed_elements);
         }
         Ok(elements)
     }
 
-    fn adjust_source_positions<'b>(elements: impl Iterator<Item = &'b mut MarkdownElement>, lines_offset: usize) {
-        for element in elements {
-            let position = match element {
-                MarkdownElement::FrontMatter(_)
-                | MarkdownElement::SetexHeading { .. }
-                | MarkdownElement::Heading { .. }
-                | MarkdownElement::Paragraph(_)
-                | MarkdownElement::List(_)
-                | MarkdownElement::Table(_)
-                | MarkdownElement::ThematicBreak
-                | MarkdownElement::BlockQuote(_) => continue,
-                MarkdownElement::Comment { source_position, .. }
-                | MarkdownElement::Snippet { source_position, .. }
-                | MarkdownElement::Image { source_position, .. } => source_position,
-            };
-            *position = position.offset_lines(lines_offset);
-        }
-    }
-
     fn parse_node(&self, node: &'a AstNode<'a>) -> ParseResult<Vec<MarkdownElement>> {
         let data = node.data.borrow();
         let element = match &data.value {
@@ -100,15 +75,16 @@ impl<'a> MarkdownParser<'a> {
             NodeValue::Table(_) => self.parse_table(node)?,
             NodeValue::CodeBlock(block) => Self::parse_code_block(block, data.sourcepos)?,
             NodeValue::ThematicBreak => MarkdownElement::ThematicBreak,
-            NodeValue::HtmlBlock(block) => Self::parse_html_block(block, data.sourcepos)?,
+            NodeValue::HtmlBlock(block) => self.parse_html_block(block, data.sourcepos)?,
             NodeValue::BlockQuote | NodeValue::MultilineBlockQuote(_) => self.parse_block_quote(node)?,
+            NodeValue::Alert(alert) => self.parse_alert(alert, node)?,
             other => return Err(ParseErrorKind::UnsupportedElement(other.identifier()).with_sourcepos(data.sourcepos)),
         };
         Ok(vec![element])
     }
 
     fn parse_front_matter(contents: &str) -> ParseResult<MarkdownElement> {
-        // Remote leading and trailing delimiters before parsing. This is quite poopy but hey, it
+        // Remove leading and trailing delimiters before parsing. This is quite poopy but hey, it
         // works.
         let contents = contents.strip_prefix("---\n").unwrap_or(contents);
         let contents = contents.strip_prefix("---\r\n").unwrap_or(contents);
@@ -119,7 +95,7 @@ impl<'a> MarkdownParser<'a> {
         Ok(MarkdownElement::FrontMatter(contents.into()))
     }
 
-    fn parse_html_block(block: &NodeHtmlBlock, sourcepos: Sourcepos) -> ParseResult<MarkdownElement> {
+    fn parse_html_block(&self, block: &NodeHtmlBlock, sourcepos: Sourcepos) -> ParseResult<MarkdownElement> {
         let block = block.literal.trim();
         let start_tag = "<!--";
         let end_tag = "-->";
@@ -132,19 +108,19 @@ impl<'a> MarkdownParser<'a> {
     }
 
     fn parse_block_quote(&self, node: &'a AstNode<'a>) -> ParseResult<MarkdownElement> {
-        let mut elements = Vec::new();
+        let mut lines = Vec::new();
         let inlines = InlinesParser::new(self.arena, SoftBreak::Newline, StringifyImages::Yes).parse(node)?;
         for inline in inlines {
             match inline {
-                Inline::Text(text) => elements.push(text),
-                Inline::LineBreak => elements.push(TextBlock::from("")),
+                Inline::Text(text) => lines.push(text),
+                Inline::LineBreak => lines.push(Line::from("")),
                 Inline::Image { .. } => {}
             }
         }
-        if elements.last() == Some(&TextBlock::from("")) {
-            elements.pop();
+        if lines.last() == Some(&Line::from("")) {
+            lines.pop();
         }
-        Ok(MarkdownElement::BlockQuote(elements))
+        Ok(MarkdownElement::BlockQuote(lines))
     }
 
     fn parse_code_block(block: &NodeCodeBlock, sourcepos: Sourcepos) -> ParseResult<MarkdownElement> {
@@ -158,6 +134,11 @@ impl<'a> MarkdownParser<'a> {
         })
     }
 
+    fn parse_alert(&self, alert: &NodeAlert, node: &'a AstNode<'a>) -> ParseResult<MarkdownElement> {
+        let MarkdownElement::BlockQuote(lines) = self.parse_block_quote(node)? else { panic!("not a block quote") };
+        Ok(MarkdownElement::Alert { alert_type: alert.alert_type, title: alert.title.clone(), lines })
+    }
+
     fn parse_heading(&self, heading: &NodeHeading, node: &'a AstNode<'a>) -> ParseResult<MarkdownElement> {
         let text = self.parse_text(node)?;
         if heading.setext {
@@ -193,7 +174,7 @@ impl<'a> MarkdownParser<'a> {
         Ok(elements)
     }
 
-    fn parse_text(&self, node: &'a AstNode<'a>) -> ParseResult<TextBlock> {
+    fn parse_text(&self, node: &'a AstNode<'a>) -> ParseResult<Line> {
         let inlines = InlinesParser::new(self.arena, SoftBreak::Space, StringifyImages::No).parse(node)?;
         let mut chunks = Vec::new();
         for inline in inlines {
@@ -205,7 +186,7 @@ impl<'a> MarkdownParser<'a> {
                 }
             };
         }
-        Ok(TextBlock(chunks))
+        Ok(Line(chunks))
     }
 
     fn parse_list(&self, root: &'a AstNode<'a>, depth: u8) -> ParseResult<Vec<ListItem>> {
@@ -329,11 +310,16 @@ impl<'a> InlinesParser<'a> {
     fn store_pending_text(&mut self) {
         let chunks = mem::take(&mut self.pending_text);
         if !chunks.is_empty() {
-            self.inlines.push(Inline::Text(TextBlock(chunks)));
+            self.inlines.push(Inline::Text(Line(chunks)));
         }
     }
 
-    fn process_node(&mut self, node: &'a AstNode<'a>, parent: &'a AstNode<'a>, style: TextStyle) -> ParseResult<()> {
+    fn process_node(
+        &mut self,
+        node: &'a AstNode<'a>,
+        parent: &'a AstNode<'a>,
+        style: TextStyle,
+    ) -> ParseResult<Option<HtmlStyle>> {
         let data = node.data.borrow();
         match &data.value {
             NodeValue::Text(text) => {
@@ -350,7 +336,7 @@ impl<'a> InlinesParser<'a> {
                     SoftBreak::Newline => {
                         self.store_pending_text();
                     }
-                    SoftBreak::Space => self.pending_text.push(Text::from(" ")),
+                    SoftBreak::Space => self.pending_text.push(Text::new(" ", style)),
                 };
             }
             NodeValue::Link(link) => {
@@ -376,7 +362,7 @@ impl<'a> InlinesParser<'a> {
             NodeValue::Image(link) => {
                 if matches!(self.stringify_images, StringifyImages::Yes) {
                     self.pending_text.push(Text::from(format!("![{}]({})", link.title, link.url)));
-                    return Ok(());
+                    return Ok(None);
                 }
                 self.store_pending_text();
 
@@ -419,24 +405,51 @@ impl<'a> InlinesParser<'a> {
                 };
                 self.process_children(node, style)?;
             }
+            NodeValue::HtmlInline(html) => {
+                let html_inline = HtmlParser::default()
+                    .parse(html)
+                    .map_err(|e| ParseErrorKind::InvalidHtml(e).with_sourcepos(data.sourcepos))?;
+                match html_inline {
+                    HtmlInline::OpenSpan { style } => return Ok(Some(HtmlStyle::Add(style))),
+                    HtmlInline::CloseSpan => return Ok(Some(HtmlStyle::Remove)),
+                };
+            }
             other => {
                 return Err(ParseErrorKind::UnsupportedStructure { container: "text", element: other.identifier() }
                     .with_sourcepos(data.sourcepos));
             }
         };
-        Ok(())
+        Ok(None)
     }
 
-    fn process_children(&mut self, root: &'a AstNode<'a>, style: TextStyle) -> ParseResult<()> {
+    fn process_children(&mut self, root: &'a AstNode<'a>, base_style: TextStyle) -> ParseResult<()> {
+        let mut html_styles = Vec::new();
+        let mut style = base_style;
         for node in root.children() {
-            self.process_node(node, root, style)?;
+            if let Some(html_style) = self.process_node(node, root, style)? {
+                match html_style {
+                    HtmlStyle::Add(style) => html_styles.push(style),
+                    HtmlStyle::Remove => {
+                        html_styles.pop();
+                    }
+                };
+                style = base_style;
+                for html_style in html_styles.iter().rev() {
+                    style.merge(html_style);
+                }
+            }
         }
         Ok(())
     }
 }
 
+enum HtmlStyle {
+    Add(TextStyle),
+    Remove,
+}
+
 enum Inline {
-    Text(TextBlock),
+    Text(Line),
     Image { path: String, title: String },
     LineBreak,
 }
@@ -463,7 +476,7 @@ pub struct ParseError {
 
 impl Display for ParseError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(f, "parse error at {}:{}: {}", self.sourcepos.start.line, self.sourcepos.start.column, self.kind)
+        write!(f, "parse error at {}: {}", self.sourcepos, self.kind)
     }
 }
 
@@ -485,6 +498,9 @@ pub(crate) enum ParseErrorKind {
     /// We don't support unfenced code blocks.
     UnfencedCodeBlock,
 
+    /// Invalid HTML was found.
+    InvalidHtml(ParseHtmlError),
+
     /// An internal parsing error.
     Internal(String),
 }
@@ -497,6 +513,7 @@ impl Display for ParseErrorKind {
                 write!(f, "unsupported structure in {container}: {element}")
             }
             Self::UnfencedCodeBlock => write!(f, "only fenced code blocks are supported"),
+            Self::InvalidHtml(inner) => write!(f, "invalid HTML: {inner}"),
             Self::Internal(message) => write!(f, "internal error: {message}"),
         }
     }
@@ -553,6 +570,9 @@ impl Identifier for NodeValue {
             NodeValue::Underline => "underline",
             NodeValue::SpoileredText => "spoilered text",
             NodeValue::EscapedTag(_) => "escaped tag",
+            NodeValue::Subscript => "subscript",
+            NodeValue::Raw(_) => "raw",
+            NodeValue::Alert(_) => "alert",
         }
     }
 }
@@ -560,6 +580,7 @@ impl Identifier for NodeValue {
 #[cfg(test)]
 mod test {
     use super::*;
+    use crate::markdown::text_style::Color;
     use rstest::rstest;
     use std::path::Path;
 
@@ -609,7 +630,24 @@ boop
             Text::new("strikethrough", TextStyle::default().strikethrough()),
         ];
 
-        let expected_elements = &[TextBlock(expected_chunks)];
+        let expected_elements = &[Line(expected_chunks)];
+        assert_eq!(elements, expected_elements);
+    }
+
+    #[test]
+    fn html_inlines() {
+        let parsed = parse_single(
+            "hi<span style=\"color: red\">red<span style=\"background-color: blue\">blue<span style=\"color: yellow\">yellow</span></span></span>",
+        );
+        let MarkdownElement::Paragraph(elements) = parsed else { panic!("not a paragraph: {parsed:?}") };
+        let expected_chunks = vec![
+            Text::from("hi"),
+            Text::new("red", TextStyle::default().fg_color(Color::Red)),
+            Text::new("blue", TextStyle::default().fg_color(Color::Red).bg_color(Color::Blue)),
+            Text::new("yellow", TextStyle::default().fg_color(Color::Yellow).bg_color(Color::Blue)),
+        ];
+
+        let expected_elements = &[Line(expected_chunks)];
         assert_eq!(elements, expected_elements);
     }
 
@@ -620,7 +658,7 @@ boop
         let expected_chunks =
             vec![Text::from("my "), Text::new("https://example.com", TextStyle::default().link_url())];
 
-        let expected_elements = &[TextBlock(expected_chunks)];
+        let expected_elements = &[Line(expected_chunks)];
         assert_eq!(elements, expected_elements);
     }
 
@@ -636,7 +674,7 @@ boop
             Text::from(")"),
         ];
 
-        let expected_elements = &[TextBlock(expected_chunks)];
+        let expected_elements = &[Line(expected_chunks)];
         assert_eq!(elements, expected_elements);
     }
 
@@ -652,7 +690,7 @@ boop
             Text::from("\""),
         ];
 
-        let expected_elements = &[TextBlock(expected_chunks)];
+        let expected_elements = &[Line(expected_chunks)];
         assert_eq!(elements, expected_elements);
     }
 
@@ -671,7 +709,7 @@ boop
             Text::from(")"),
         ];
 
-        let expected_elements = &[TextBlock(expected_chunks)];
+        let expected_elements = &[Line(expected_chunks)];
         assert_eq!(elements, expected_elements);
     }
 
@@ -844,21 +882,21 @@ let q = 42;
         assert_eq!(lines.len(), 11);
         assert_eq!(
             lines[0],
-            TextBlock(vec![Text::from("foo "), Text::new("is not", TextStyle::default().bold()), Text::from(" bar")])
+            Line(vec![Text::from("foo "), Text::new("is not", TextStyle::default().bold()), Text::from(" bar")])
         );
         assert_eq!(
             lines[1],
-            TextBlock(vec![Text::from("![](hehe.png)"), Text::from(" test "), Text::from("![](potato.png)")])
+            Line(vec![Text::from("![](hehe.png)"), Text::from(" test "), Text::from("![](potato.png)")])
         );
-        assert_eq!(lines[2], TextBlock::from(""));
-        assert_eq!(lines[3], TextBlock(vec![Text::from("* "), Text::from("a")]));
-        assert_eq!(lines[4], TextBlock(vec![Text::from("* "), Text::from("b")]));
-        assert_eq!(lines[5], TextBlock::from(""));
-        assert_eq!(lines[6], TextBlock(vec![Text::from("1. "), Text::from("a")]));
-        assert_eq!(lines[7], TextBlock(vec![Text::from("2. "), Text::from("b")]));
-        assert_eq!(lines[8], TextBlock::from(""));
-        assert_eq!(lines[9], TextBlock(vec![Text::from("1) "), Text::from("a")]));
-        assert_eq!(lines[10], TextBlock(vec![Text::from("2) "), Text::from("b")]));
+        assert_eq!(lines[2], Line::from(""));
+        assert_eq!(lines[3], Line(vec![Text::from("* "), Text::from("a")]));
+        assert_eq!(lines[4], Line(vec![Text::from("* "), Text::from("b")]));
+        assert_eq!(lines[5], Line::from(""));
+        assert_eq!(lines[6], Line(vec![Text::from("1. "), Text::from("a")]));
+        assert_eq!(lines[7], Line(vec![Text::from("2. "), Text::from("b")]));
+        assert_eq!(lines[8], Line::from(""));
+        assert_eq!(lines[9], Line(vec![Text::from("1) "), Text::from("a")]));
+        assert_eq!(lines[10], Line(vec![Text::from("2) "), Text::from("b")]));
     }
 
     #[test]
@@ -875,11 +913,11 @@ foo
         );
         let MarkdownElement::BlockQuote(lines) = parsed else { panic!("not a block quote: {parsed:?}") };
         assert_eq!(lines.len(), 5);
-        assert_eq!(lines[0], TextBlock::from("bar"));
-        assert_eq!(lines[1], TextBlock::from("foo"));
-        assert_eq!(lines[2], TextBlock::from(""));
-        assert_eq!(lines[3], TextBlock(vec![Text::from("* "), Text::from("a")]));
-        assert_eq!(lines[4], TextBlock(vec![Text::from("* "), Text::from("b")]));
+        assert_eq!(lines[0], Line::from("bar"));
+        assert_eq!(lines[1], Line::from("foo"));
+        assert_eq!(lines[2], Line::from(""));
+        assert_eq!(lines[3], Line(vec![Text::from("* "), Text::from("a")]));
+        assert_eq!(lines[4], Line(vec![Text::from("* "), Text::from("b")]));
     }
 
     #[test]
@@ -911,7 +949,7 @@ mom
         let Err(e) = result else {
             panic!("parsing didn't fail");
         };
-        assert_eq!(e.sourcepos.start.line, 5);
+        assert_eq!(e.sourcepos.start.line, 6);
         assert_eq!(e.sourcepos.start.column, 3);
     }
 
@@ -927,7 +965,7 @@ mom
 ",
         );
         let MarkdownElement::Comment { source_position, .. } = &parsed[1] else { panic!("not a comment") };
-        assert_eq!(source_position.start.line, 5);
+        assert_eq!(source_position.start.line, 6);
         assert_eq!(source_position.start.column, 1);
     }
 
@@ -942,4 +980,17 @@ mom
         let expected = format!("hi{nl}mom{nl}");
         assert_eq!(contents, &expected);
     }
+
+    #[test]
+    fn parse_alert() {
+        let input = r"
+> [!note]
+> hi mom
+> bye **mom**
+";
+        let MarkdownElement::Alert { lines, .. } = parse_single(&input) else {
+            panic!("not an alert");
+        };
+        assert_eq!(lines.len(), 2);
+    }
 }
