@@ -1,19 +1,23 @@
 use crate::{
-    custom::KeyBindingsConfig,
-    diff::PresentationDiffer,
-    execute::SnippetExecutor,
+    code::execute::SnippetExecutor,
+    commands::{
+        listener::{Command, CommandListener},
+        speaker_notes::{SpeakerNotesEvent, SpeakerNotesEventPublisher},
+    },
+    config::KeyBindingsConfig,
     export::ImageReplacer,
-    input::source::{Command, CommandSource},
     markdown::parse::{MarkdownParser, ParseError},
-    media::{printer::ImagePrinter, register::ImageRegistry},
-    presentation::{Presentation, RenderAsyncState},
-    processing::builder::{BuildError, PresentationBuilder, PresentationBuilderOptions, Themes},
+    presentation::{
+        Presentation,
+        builder::{BuildError, PresentationBuilder, PresentationBuilderOptions, Themes},
+        diff::PresentationDiffer,
+    },
     render::{
-        draw::{ErrorSource, RenderError, RenderResult, TerminalDrawer},
-        properties::WindowSize,
-        validate::OverflowValidator,
+        ErrorSource, RenderError, RenderResult, TerminalDrawer, TerminalDrawerOptions, operation::RenderAsyncState,
+        properties::WindowSize, validate::OverflowValidator,
     },
     resource::Resources,
+    terminal::image::printer::{ImagePrinter, ImageRegistry},
     theme::PresentationTheme,
     third_party::ThirdPartyRender,
 };
@@ -35,6 +39,7 @@ pub struct PresenterOptions {
     pub font_size_fallback: u8,
     pub bindings: KeyBindingsConfig,
     pub validate_overflows: bool,
+    pub max_columns: u16,
 }
 
 /// A slideshow presenter.
@@ -42,7 +47,7 @@ pub struct PresenterOptions {
 /// This type puts everything else together.
 pub struct Presenter<'a> {
     default_theme: &'a PresentationTheme,
-    commands: CommandSource,
+    listener: CommandListener,
     parser: MarkdownParser<'a>,
     resources: Resources,
     third_party: ThirdPartyRender,
@@ -52,6 +57,7 @@ pub struct Presenter<'a> {
     image_printer: Arc<ImagePrinter>,
     themes: Themes,
     options: PresenterOptions,
+    speaker_notes_event_publisher: Option<SpeakerNotesEventPublisher>,
 }
 
 impl<'a> Presenter<'a> {
@@ -59,7 +65,7 @@ impl<'a> Presenter<'a> {
     #[allow(clippy::too_many_arguments)]
     pub fn new(
         default_theme: &'a PresentationTheme,
-        commands: CommandSource,
+        listener: CommandListener,
         parser: MarkdownParser<'a>,
         resources: Resources,
         third_party: ThirdPartyRender,
@@ -67,10 +73,11 @@ impl<'a> Presenter<'a> {
         themes: Themes,
         image_printer: Arc<ImagePrinter>,
         options: PresenterOptions,
+        speaker_notes_event_publisher: Option<SpeakerNotesEventPublisher>,
     ) -> Self {
         Self {
             default_theme,
-            commands,
+            listener,
             parser,
             resources,
             third_party,
@@ -80,6 +87,7 @@ impl<'a> Presenter<'a> {
             image_printer,
             themes,
             options,
+            speaker_notes_event_publisher,
         }
     }
 
@@ -91,8 +99,11 @@ impl<'a> Presenter<'a> {
         self.state = PresenterState::Presenting(Presentation::from(vec![]));
         self.try_reload(path, true);
 
-        let mut drawer =
-            TerminalDrawer::new(io::stdout(), self.image_printer.clone(), self.options.font_size_fallback)?;
+        let drawer_options = TerminalDrawerOptions {
+            font_size_fallback: self.options.font_size_fallback,
+            max_columns: self.options.max_columns,
+        };
+        let mut drawer = TerminalDrawer::new(io::stdout(), self.image_printer.clone(), drawer_options)?;
         loop {
             if matches!(self.options.mode, PresentMode::Export) {
                 if let PresenterState::Failure { error, .. } = &self.state {
@@ -108,7 +119,7 @@ impl<'a> Presenter<'a> {
                     self.render(&mut drawer)?;
                 }
 
-                let command = match self.commands.try_next_command()? {
+                let command = match self.listener.try_next_command()? {
                     Some(command) => command,
                     _ => match self.resources.resources_modified() {
                         true => Command::Reload,
@@ -121,7 +132,10 @@ impl<'a> Presenter<'a> {
                     },
                 };
                 match self.apply_command(command) {
-                    CommandSideEffect::Exit => return Ok(()),
+                    CommandSideEffect::Exit => {
+                        self.publish_event(SpeakerNotesEvent::Exit)?;
+                        return Ok(());
+                    }
                     CommandSideEffect::Suspend => {
                         self.suspend(&mut drawer);
                         break;
@@ -136,9 +150,19 @@ impl<'a> Presenter<'a> {
                     CommandSideEffect::None => (),
                 };
             }
+            self.publish_event(SpeakerNotesEvent::GoToSlide {
+                slide: self.state.presentation().current_slide_index() as u32 + 1,
+            })?;
         }
     }
 
+    fn publish_event(&self, event: SpeakerNotesEvent) -> io::Result<()> {
+        if let Some(publisher) = &self.speaker_notes_event_publisher {
+            publisher.send(event)?;
+        }
+        Ok(())
+    }
+
     fn check_async_error(&mut self) -> bool {
         let error_holder = self.state.presentation().state.async_error_holder();
         let error_holder = error_holder.lock().unwrap();
@@ -180,14 +204,16 @@ impl<'a> Presenter<'a> {
 
     fn render(&mut self, drawer: &mut TerminalDrawer<Stdout>) -> RenderResult {
         let result = match &self.state {
-            PresenterState::Presenting(presentation) => drawer.render_slide(presentation),
+            PresenterState::Presenting(presentation) => {
+                drawer.render_operations(presentation.current_slide().iter_visible_operations())
+            }
             PresenterState::SlideIndex(presentation) => {
-                drawer.render_slide(presentation)?;
-                drawer.render_slide_index(presentation)
+                drawer.render_operations(presentation.current_slide().iter_visible_operations())?;
+                drawer.render_operations(presentation.iter_slide_index_operations())
             }
             PresenterState::KeyBindings(presentation) => {
-                drawer.render_slide(presentation)?;
-                drawer.render_key_bindings(presentation)
+                drawer.render_operations(presentation.current_slide().iter_visible_operations())?;
+                drawer.render_operations(presentation.iter_bindings_operations())
             }
             PresenterState::Failure { error, source, .. } => drawer.render_error(error, source),
             PresenterState::Empty => panic!("cannot render without state"),
