@@ -1,14 +1,11 @@
-use super::terminal::{Terminal, TerminalWrite};
 use crate::{
     markdown::{
         elements::Text,
-        text::{WeightedText, WeightedTextBlock},
+        text::{WeightedLine, WeightedText},
+        text_style::{Color, Colors},
     },
-    render::{
-        draw::{RenderError, RenderResult},
-        layout::Positioning,
-    },
-    style::{Color, Colors},
+    render::{RenderError, RenderResult, layout::Positioning},
+    terminal::{Terminal, TerminalWrite},
 };
 
 const MINIMUM_LINE_LENGTH: u16 = 10;
@@ -19,7 +16,7 @@ const MINIMUM_LINE_LENGTH: u16 = 10;
 pub(crate) struct TextDrawer<'a> {
     prefix: &'a WeightedText,
     right_padding_length: u16,
-    line: &'a WeightedTextBlock,
+    line: &'a WeightedLine,
     positioning: Positioning,
     prefix_length: u16,
     default_colors: &'a Colors,
@@ -32,7 +29,7 @@ impl<'a> TextDrawer<'a> {
     pub(crate) fn new(
         prefix: &'a WeightedText,
         right_padding_length: u16,
-        line: &'a WeightedTextBlock,
+        line: &'a WeightedLine,
         positioning: Positioning,
         default_colors: &'a Colors,
     ) -> Result<Self, RenderError> {
@@ -86,7 +83,7 @@ impl<'a> TextDrawer<'a> {
         // Print the prefix at the beginning of the line.
         let styled_prefix = {
             let Text { content, style } = self.prefix.text();
-            style.apply(content)
+            style.apply(content)?
         };
         terminal.move_to_column(self.positioning.start_column)?;
         terminal.print_styled_line(styled_prefix.clone())?;
@@ -114,12 +111,12 @@ impl<'a> TextDrawer<'a> {
                 line_length = line_length.saturating_add(chunk.width() as u16);
 
                 let (text, style) = chunk.into_parts();
-                let text = style.apply(text);
+                let text = style.apply(text)?;
                 terminal.print_styled_line(text)?;
 
                 // Crossterm resets colors if any attributes are set so let's just re-apply colors
                 // if the format has anything on it at all.
-                if style.has_modifiers() {
+                if style != Default::default() {
                     terminal.set_colors(*self.default_colors)?;
                 }
             }
