@@ -1,6 +1,6 @@
-use crate::style::{Color, Colors};
+use crate::markdown::text_style::{Color, Colors, FixedStr, UndefinedPaletteColorError};
 use serde::{Deserialize, Serialize};
-use std::{collections::BTreeMap, fs, io, path::Path};
+use std::{collections::BTreeMap, fmt, fs, io, marker::PhantomData, path::Path};
 
 include!(concat!(env!("OUT_DIR"), "/themes.rs"));
 
@@ -12,11 +12,15 @@ pub struct PresentationThemeSet {
 impl PresentationThemeSet {
     /// Loads a theme from its name.
     pub fn load_by_name(&self, name: &str) -> Option<PresentationTheme> {
-        if let Some(contents) = THEMES.get(name) {
-            // This is going to be caught by the test down here.
-            Some(serde_yaml::from_slice(contents).expect("corrupted theme"))
-        } else {
-            self.custom_themes.get(name).cloned()
+        match THEMES.get(name) {
+            Some(contents) => {
+                // This is going to be caught by the test down here.
+                let mut theme: PresentationTheme = serde_yaml::from_slice(contents).expect("corrupted theme");
+                // SAFETY: we enforce themes are well formed before getting here
+                theme.resolve_palette_colors().expect("failed to resolve colors");
+                Some(theme)
+            }
+            None => self.custom_themes.get(name).cloned(),
         }
     }
 
@@ -39,7 +43,10 @@ impl PresentationThemeSet {
                 if THEMES.contains_key(theme_name) {
                     return Err(LoadThemeError::Duplicate(theme_name.into()));
                 }
-                let theme = PresentationTheme::from_path(entry.path())?;
+                let mut theme = PresentationTheme::from_path(entry.path())?;
+                theme
+                    .resolve_palette_colors()
+                    .map_err(|e| LoadThemeError::Corrupted(theme_name.to_string(), e.into()))?;
                 let base = theme.extends.clone();
                 self.custom_themes.insert(theme_name.into(), theme);
                 dependencies.insert(theme_name.to_string(), base);
@@ -150,6 +157,10 @@ pub struct PresentationTheme {
     #[serde(default)]
     pub(crate) block_quote: BlockQuoteStyle,
 
+    /// The style for an alert.
+    #[serde(default)]
+    pub(crate) alert: AlertStyle,
+
     /// The default style.
     #[serde(rename = "default", default)]
     pub(crate) default_style: DefaultStyle,
@@ -177,6 +188,10 @@ pub struct PresentationTheme {
     /// The style for modals.
     #[serde(default)]
     pub(crate) modals: ModalStyle,
+
+    /// The color palette.
+    #[serde(default)]
+    pub(crate) palette: ColorPalette,
 }
 
 impl PresentationTheme {
@@ -215,6 +230,43 @@ impl PresentationTheme {
         };
         alignment.clone().unwrap_or_default()
     }
+
+    pub(crate) fn resolve_palette_colors(&mut self) -> Result<(), UndefinedPaletteColorError> {
+        let Self {
+            slide_title,
+            code,
+            execution_output,
+            inline_code,
+            block_quote,
+            alert,
+            default_style,
+            headings,
+            intro_slide,
+            footer,
+            typst,
+            mermaid,
+            modals,
+            table: _,
+            palette: _,
+            extends: _,
+        } = self;
+        slide_title.resolve_palette_colors(&self.palette)?;
+        code.resolve_palette_colors(&self.palette)?;
+        execution_output.resolve_palette_colors(&self.palette)?;
+        inline_code.resolve_palette_colors(&self.palette)?;
+        block_quote.resolve_palette_colors(&self.palette)?;
+        alert.resolve_palette_colors(&self.palette)?;
+        default_style.resolve_palette_colors(&self.palette)?;
+        headings.resolve_palette_colors(&self.palette)?;
+        intro_slide.resolve_palette_colors(&self.palette)?;
+        if let Some(footer) = footer.as_mut() {
+            footer.resolve_palette_colors(&self.palette)?;
+        }
+        typst.resolve_palette_colors(&self.palette)?;
+        mermaid.resolve_palette_colors(&self.palette)?;
+        modals.resolve_palette_colors(&self.palette)?;
+        Ok(())
+    }
 }
 
 /// The style of a slide title.
@@ -253,6 +305,13 @@ pub(crate) struct SlideTitleStyle {
     pub(crate) underlined: Option<bool>,
 }
 
+impl SlideTitleStyle {
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        self.colors = self.colors.resolve(palette)?;
+        Ok(())
+    }
+}
+
 /// The style for all headings.
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub(crate) struct HeadingStyles {
@@ -281,6 +340,16 @@ pub(crate) struct HeadingStyles {
     pub(crate) h6: HeadingStyle,
 }
 
+impl HeadingStyles {
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        let Self { h1, h2, h3, h4, h5, h6 } = self;
+        for h in [h1, h2, h3, h4, h5, h6] {
+            h.resolve_palette_colors(palette)?;
+        }
+        Ok(())
+    }
+}
+
 /// The style for a heading.
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub(crate) struct HeadingStyle {
@@ -299,6 +368,13 @@ pub(crate) struct HeadingStyle {
     pub(crate) colors: Colors,
 }
 
+impl HeadingStyle {
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        self.colors = self.colors.resolve(palette)?;
+        Ok(())
+    }
+}
+
 /// The style of a block quote.
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub(crate) struct BlockQuoteStyle {
@@ -317,6 +393,14 @@ pub(crate) struct BlockQuoteStyle {
     pub(crate) colors: BlockQuoteColors,
 }
 
+impl BlockQuoteStyle {
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        let Self { colors, alignment: _, prefix: _ } = self;
+        colors.resolve_palette_colors(palette)?;
+        Ok(())
+    }
+}
+
 /// The colors of a block quote.
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub(crate) struct BlockQuoteColors {
@@ -325,9 +409,235 @@ pub(crate) struct BlockQuoteColors {
     pub(crate) base: Colors,
 
     /// The color of the vertical bar that prefixes each line in the quote.
+    #[serde(default)]
     pub(crate) prefix: Option<Color>,
 }
 
+impl BlockQuoteColors {
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        let Self { base, prefix } = self;
+        *base = base.resolve(palette)?;
+        if let Some(prefix) = prefix.as_mut() {
+            *prefix = prefix.resolve(palette)?;
+        }
+        Ok(())
+    }
+}
+
+/// The style of an alert.
+#[derive(Clone, Debug, Default, Deserialize, Serialize)]
+pub(crate) struct AlertStyle {
+    /// The alignment.
+    #[serde(flatten, default)]
+    pub(crate) alignment: Option<Alignment>,
+
+    /// The base colors.
+    #[serde(default)]
+    pub(crate) base_colors: Colors,
+
+    /// The prefix to be added to this block quote.
+    ///
+    /// This allows adding something like a vertical bar before the text.
+    #[serde(default)]
+    pub(crate) prefix: Option<String>,
+
+    /// The style for each alert type.
+    #[serde(default)]
+    pub(crate) styles: AlertTypeStyles,
+}
+
+impl AlertStyle {
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        let Self { base_colors, styles, alignment: _, prefix: _ } = self;
+        *base_colors = base_colors.resolve(palette)?;
+        styles.resolve_palette_colors(palette)?;
+        Ok(())
+    }
+}
+
+/// The style for each alert type.
+#[derive(Clone, Debug, Default, Deserialize, Serialize)]
+pub(crate) struct AlertTypeStyles {
+    /// The style for note alert types.
+    #[serde(default)]
+    pub(crate) note: AlertTypeStyle<NoteAlertType>,
+
+    /// The style for tip alert types.
+    #[serde(default)]
+    pub(crate) tip: AlertTypeStyle<TipAlertType>,
+
+    /// The style for important alert types.
+    #[serde(default)]
+    pub(crate) important: AlertTypeStyle<ImportantAlertType>,
+
+    /// The style for warning alert types.
+    #[serde(default)]
+    pub(crate) warning: AlertTypeStyle<WarningAlertType>,
+
+    /// The style for caution alert types.
+    #[serde(default)]
+    pub(crate) caution: AlertTypeStyle<CautionAlertType>,
+}
+
+impl AlertTypeStyles {
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        let Self { note, tip, important, warning, caution } = self;
+        note.resolve_palette_colors(palette)?;
+        tip.resolve_palette_colors(palette)?;
+        important.resolve_palette_colors(palette)?;
+        warning.resolve_palette_colors(palette)?;
+        caution.resolve_palette_colors(palette)?;
+        Ok(())
+    }
+}
+
+/// The style for an alert type.
+#[derive(Deserialize, Serialize)]
+pub(crate) struct AlertTypeStyle<S: AlertTypeProperties> {
+    /// The color to be used.
+    #[serde(default)]
+    pub(crate) color: Option<Color>,
+
+    /// The title to be used.
+    #[serde(default)]
+    pub(crate) title: Option<String>,
+
+    /// The icon to be used.
+    #[serde(default)]
+    pub(crate) icon: Option<String>,
+
+    #[serde(skip)]
+    _unused: PhantomData<S>,
+}
+
+impl<S: AlertTypeProperties> Default for AlertTypeStyle<S> {
+    fn default() -> Self {
+        Self {
+            color: Default::default(),
+            title: Default::default(),
+            icon: Default::default(),
+            _unused: Default::default(),
+        }
+    }
+}
+
+impl<S: AlertTypeProperties> AlertTypeStyle<S> {
+    pub(crate) fn as_parts(&self) -> (Color, &str, &str) {
+        (
+            self.color.unwrap_or(S::default_color()),
+            self.title.as_deref().unwrap_or(S::default_title()),
+            self.icon.as_deref().unwrap_or(S::default_icon()),
+        )
+    }
+
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        let Self { color, title: _, icon: _, _unused: _ } = self;
+        if let Some(color) = color {
+            *color = color.resolve(palette)?;
+        }
+        Ok(())
+    }
+}
+
+impl<S: AlertTypeProperties> fmt::Debug for AlertTypeStyle<S> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("AlertTypeStyle")
+            .field("color", &self.color)
+            .field("title", &self.title)
+            .field("icon", &self.icon)
+            .field("_unused", &self._unused)
+            .finish()
+    }
+}
+
+impl<S: AlertTypeProperties> Clone for AlertTypeStyle<S> {
+    fn clone(&self) -> Self {
+        Self { color: self.color, title: self.title.clone(), icon: self.icon.clone(), _unused: PhantomData }
+    }
+}
+
+pub(crate) trait AlertTypeProperties {
+    fn default_title() -> &'static str;
+    fn default_icon() -> &'static str;
+    fn default_color() -> Color;
+}
+
+pub(crate) struct NoteAlertType;
+pub(crate) struct TipAlertType;
+pub(crate) struct ImportantAlertType;
+pub(crate) struct WarningAlertType;
+pub(crate) struct CautionAlertType;
+
+impl AlertTypeProperties for NoteAlertType {
+    fn default_title() -> &'static str {
+        "Note"
+    }
+
+    fn default_icon() -> &'static str {
+        "󰋽"
+    }
+
+    fn default_color() -> Color {
+        Color::Blue
+    }
+}
+
+impl AlertTypeProperties for TipAlertType {
+    fn default_title() -> &'static str {
+        "Tip"
+    }
+
+    fn default_icon() -> &'static str {
+        ""
+    }
+
+    fn default_color() -> Color {
+        Color::Green
+    }
+}
+
+impl AlertTypeProperties for ImportantAlertType {
+    fn default_title() -> &'static str {
+        "Important"
+    }
+
+    fn default_icon() -> &'static str {
+        ""
+    }
+
+    fn default_color() -> Color {
+        Color::Cyan
+    }
+}
+
+impl AlertTypeProperties for WarningAlertType {
+    fn default_title() -> &'static str {
+        "Warning"
+    }
+
+    fn default_icon() -> &'static str {
+        ""
+    }
+
+    fn default_color() -> Color {
+        Color::Yellow
+    }
+}
+
+impl AlertTypeProperties for CautionAlertType {
+    fn default_title() -> &'static str {
+        "Caution"
+    }
+
+    fn default_icon() -> &'static str {
+        "󰳦"
+    }
+
+    fn default_color() -> Color {
+        Color::Red
+    }
+}
+
 /// The style for the presentation introduction slide.
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub(crate) struct IntroSlideStyle {
@@ -360,6 +670,17 @@ pub(crate) struct IntroSlideStyle {
     pub(crate) footer: Option<bool>,
 }
 
+impl IntroSlideStyle {
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        let Self { title, subtitle, event, location, date, author, footer: _footer } = self;
+        for s in [title, subtitle, event, location, date] {
+            s.resolve_palette_colors(palette)?;
+        }
+        author.resolve_palette_colors(palette)?;
+        Ok(())
+    }
+}
+
 /// A simple style.
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub(crate) struct DefaultStyle {
@@ -372,6 +693,14 @@ pub(crate) struct DefaultStyle {
     pub(crate) colors: Colors,
 }
 
+impl DefaultStyle {
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        let Self { colors, margin: _margin } = self;
+        *colors = colors.resolve(palette)?;
+        Ok(())
+    }
+}
+
 /// A simple style.
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub(crate) struct BasicStyle {
@@ -384,6 +713,14 @@ pub(crate) struct BasicStyle {
     pub(crate) colors: Colors,
 }
 
+impl BasicStyle {
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        let Self { colors, alignment: _ } = self;
+        *colors = colors.resolve(palette)?;
+        Ok(())
+    }
+}
+
 /// Text alignment.
 ///
 /// This allows anchoring presentation elements to the left, center, or right of the screen.
@@ -438,6 +775,14 @@ pub(crate) struct AuthorStyle {
     pub(crate) positioning: AuthorPositioning,
 }
 
+impl AuthorStyle {
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        let Self { colors, alignment: _, positioning: _ } = self;
+        *colors = colors.resolve(palette)?;
+        Ok(())
+    }
+}
+
 /// The style of the footer that's shown in every slide.
 #[derive(Clone, Debug, Deserialize, Serialize)]
 #[serde(tag = "style", rename_all = "snake_case")]
@@ -472,6 +817,19 @@ pub(crate) enum FooterStyle {
     Empty,
 }
 
+impl FooterStyle {
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        use FooterStyle::*;
+        match self {
+            Template { colors, left: _, center: _, right: _ } | ProgressBar { colors, character: _ } => {
+                *colors = colors.resolve(palette)?;
+                Ok(())
+            }
+            Empty => Ok(()),
+        }
+    }
+}
+
 impl Default for FooterStyle {
     fn default() -> Self {
         Self::Template { left: None, center: None, right: None, colors: Colors::default() }
@@ -497,6 +855,13 @@ pub(crate) struct CodeBlockStyle {
     pub(crate) background: Option<bool>,
 }
 
+impl CodeBlockStyle {
+    fn resolve_palette_colors(&mut self, _: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        let Self { alignment: _, padding: _, theme_name: _, background: _ } = self;
+        Ok(())
+    }
+}
+
 /// The style for the output of a code execution block.
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub(crate) struct ExecutionOutputBlockStyle {
@@ -509,6 +874,14 @@ pub(crate) struct ExecutionOutputBlockStyle {
     pub(crate) status: ExecutionStatusBlockStyle,
 }
 
+impl ExecutionOutputBlockStyle {
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        let Self { colors, status } = self;
+        *colors = colors.resolve(palette)?;
+        status.resolve_palette_colors(palette)
+    }
+}
+
 /// The style for the status of a code execution block.
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub(crate) struct ExecutionStatusBlockStyle {
@@ -529,6 +902,16 @@ pub(crate) struct ExecutionStatusBlockStyle {
     pub(crate) not_started: Colors,
 }
 
+impl ExecutionStatusBlockStyle {
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        let Self { running, success, failure, not_started } = self;
+        for c in [running, success, failure, not_started] {
+            *c = c.resolve(palette)?;
+        }
+        Ok(())
+    }
+}
+
 /// The style for inline code.
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub(crate) struct InlineCodeStyle {
@@ -537,6 +920,14 @@ pub(crate) struct InlineCodeStyle {
     pub(crate) colors: Colors,
 }
 
+impl InlineCodeStyle {
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        let Self { colors } = self;
+        *colors = colors.resolve(palette)?;
+        Ok(())
+    }
+}
+
 /// Vertical/horizontal padding.
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub(crate) struct PaddingRect {
@@ -632,6 +1023,14 @@ pub(crate) struct TypstStyle {
     pub(crate) colors: Colors,
 }
 
+impl TypstStyle {
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        let Self { colors, horizontal_margin: _, vertical_margin: _ } = self;
+        *colors = colors.resolve(palette)?;
+        Ok(())
+    }
+}
+
 /// Mermaid styles.
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub(crate) struct MermaidStyle {
@@ -642,6 +1041,13 @@ pub(crate) struct MermaidStyle {
     pub(crate) background: Option<String>,
 }
 
+impl MermaidStyle {
+    fn resolve_palette_colors(&mut self, _: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        let Self { theme: _, background: _ } = self;
+        Ok(())
+    }
+}
+
 /// Modals style.
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub(crate) struct ModalStyle {
@@ -654,13 +1060,30 @@ pub(crate) struct ModalStyle {
     pub(crate) selection_colors: Colors,
 }
 
+impl ModalStyle {
+    fn resolve_palette_colors(&mut self, palette: &ColorPalette) -> Result<(), UndefinedPaletteColorError> {
+        let Self { colors, selection_colors } = self;
+        for c in [colors, selection_colors] {
+            *c = c.resolve(palette)?;
+        }
+        Ok(())
+    }
+}
+
+/// The color palette.
+#[derive(Clone, Debug, Default, Deserialize, Serialize)]
+pub(crate) struct ColorPalette {
+    #[serde(default)]
+    pub(crate) colors: BTreeMap<FixedStr, Color>,
+}
+
 /// An error loading a presentation theme.
 #[derive(thiserror::Error, Debug)]
 pub enum LoadThemeError {
     #[error(transparent)]
     Io(#[from] io::Error),
 
-    #[error("theme at '{0}' is corrupted: {1}")]
+    #[error("theme '{0}' is corrupted: {1}")]
     Corrupted(String, Box<dyn std::error::Error>),
 
     #[error("duplicate custom theme '{0}'")]
@@ -688,7 +1111,7 @@ mod test {
     fn validate_themes() {
         let themes = PresentationThemeSet::default();
         for theme_name in THEMES.keys() {
-            let Some(theme) = themes.load_by_name(theme_name) else {
+            let Some(mut theme) = themes.load_by_name(theme_name).clone() else {
                 panic!("theme '{theme_name}' is corrupted");
             };
 
@@ -697,6 +1120,8 @@ mod test {
 
             let merged = merge_struct::merge(&PresentationTheme::default(), &theme);
             assert!(merged.is_ok(), "theme '{theme_name}' can't be merged: {}", merged.unwrap_err());
+
+            theme.resolve_palette_colors().expect("failed to resolve palette colors");
         }
     }
 
