@@ -1,15 +1,18 @@
 use crate::{
     ImageRegistry, PresentationTheme,
-    custom::{default_mermaid_scale, default_snippet_render_threads, default_typst_ppi},
-    markdown::elements::{Percent, Text, TextBlock},
-    media::{image::Image, printer::RegisterImageError},
-    presentation::{
-        AsRenderOperations, AsyncPresentationError, AsyncPresentationErrorHolder, ImageProperties, ImageSize,
-        RenderAsync, RenderAsyncState, RenderOperation,
+    config::{default_mermaid_scale, default_snippet_render_threads, default_typst_ppi},
+    markdown::{
+        elements::{Line, Percent, Text},
+        text_style::{Color, Colors, TextStyle},
     },
-    processing::builder::DEFAULT_IMAGE_Z_INDEX,
-    render::properties::WindowSize,
-    style::{Color, Colors, TextStyle},
+    presentation::{AsyncPresentationError, AsyncPresentationErrorHolder, builder::DEFAULT_IMAGE_Z_INDEX},
+    render::{
+        operation::{
+            AsRenderOperations, ImageRenderProperties, ImageSize, RenderAsync, RenderAsyncState, RenderOperation,
+        },
+        properties::WindowSize,
+    },
+    terminal::image::{Image, printer::RegisterImageError},
     theme::{Alignment, MermaidStyle, TypstStyle},
     tools::{ExecutionError, ThirdPartyTools},
 };
@@ -355,7 +358,7 @@ impl AsRenderOperations for RenderThirdParty {
                     Some(percent) => ImageSize::WidthScaled { ratio: percent.as_ratio() },
                     None => Default::default(),
                 };
-                let properties = ImageProperties {
+                let properties = ImageRenderProperties {
                     z_index: DEFAULT_IMAGE_Z_INDEX,
                     size,
                     restore_cursor: false,
@@ -368,7 +371,7 @@ impl AsRenderOperations for RenderThirdParty {
                 ]
             }
             None => {
-                let text = TextBlock::from(Text::new("Loading...", TextStyle::default().bold()));
+                let text = Line::from(Text::new("Loading...", TextStyle::default().bold()));
                 vec![RenderOperation::RenderText {
                     line: text.into(),
                     alignment: Alignment::Center { minimum_margin: Default::default(), minimum_size: 0 },
